﻿using Newtonsoft.Json;
using ServiceCenter.Extensions;
using ServiceCenter.Redis;
using System.Collections.Concurrent;
using WCS.Core;

namespace ServiceCenter.Logs
{
    public static class LogHub
    {
        /// <summary>
        ///  发布世界交互日志
        /// </summary>
        /// <param name="logs"></param>
        /// <param name="World">当前世界</param>
        public static void WorldPublish(ConcurrentQueue<KeyLog> logs, string World)
        {
            if (logs.Count > 0)
            {
                try
                {
                    //每一条数据存入redis
                    foreach (var log in logs)
                    {
                        var dir = $"D:\\WCSLogs\\{DateTime.Now.yyyyMMdd()}\\{log.Channel.World}\\{log.Channel.System}\\{log.Channel.Item}\\";
                        var msg = $"{log.Time.yyyyMMddhhmmssf()}--[{Thread.CurrentThread.ManagedThreadId}]--{log}\n";
                        RedisHub.Default.RPush("LogHub", JsonConvert.SerializeObject(new LogModel
                        {
                            path = dir,
                            Title = $"{log.Log.Message.Split(":")[0]}.txt",
                            Con = msg
                        }));
                    }
                    //存入业务报警内容
                    List<BusinessAlarm> businesses = logs.Where(v => v.Log.Level > LogLevelEnum.Low && v.Log.LogUpLoad == LogUpLoadEnum.UpLoadWMS).Select(v => new BusinessAlarm()
                    {
                        BusinessName = v.Channel.System,
                        DevNo = v.Channel.Item,
                        Con = v.ToString(),
                        Time = DateTime.Now
                    }).ToList();
                    RedisHub.WMS.Set($"{nameof(BusinessAlarm)}:{World}", JsonConvert.SerializeObject(businesses));
                }
                catch
                {
                }
            }
        }

        /// <summary>
        ///   执行记录
        /// </summary>
        /// <param name="system">系统</param>
        /// <param name="devCode">设备号</param>
        /// <param name="msg">内容</param>
        public static async void ExRecord(this SystemBase system, string devCode, string msg)
        {
            var key = $"{system.World.Description}:{devCode}";
            RedisHub.Default.RPush($"{system.World.Description}:{devCode}", msg);
            if (RedisHub.Monitor.LLen(key) > 5000)
            {
                RedisHub.Monitor.LTrim(key, 4000, -1);
            }
        }

        /// <summary>
        ///   执行记录
        /// </summary>
        /// <param name="system">系统</param>
        /// <param name="devCode">设备号</param>
        /// <param name="msg">内容</param>
        public static async void ExRecord<T>(this SystemBase system, string devCode, string msg, List<T> ints)
        {
            try
            {
                var key = $"{system.World.Description}:{devCode}";
                RedisHub.Default.RPush(key, msg + ints.JsonToString());
                if (RedisHub.Monitor.LLen(key) > 5000)
                {
                    RedisHub.Monitor.LTrim(key, 4000, -1);
                }
            }
            catch (Exception e)
            {
                //Console.WriteLine(e);
            }
        }

        /// <summary>
        ///  发布一条日志记录
        /// </summary>
        /// <param name="title">接口名</param>
        /// <param name="con">内容</param>
        public static void InterfacePublish(string title, string con)
        {
            RedisHub.Default.RPush("LogHub", JsonConvert.SerializeObject(new LogModel
            {
                path = $"D:\\WCSLogs\\{DateTime.Now.yyyyMMdd()}\\接口日志\\",
                Title = $"{title}.txt",
                Con = $"{DateTime.Now.yyyyMMddhhmmssf()}--{con}\n"
            }));
        }

        /// <summary>
        ///  初始化日志处理进程
        /// </summary>
        public static void init()
        {
            while (true)
            {
                LogModel log = new LogModel();
                try
                {
                    var logHub = RedisHub.Default.BLPop("LogHub", 0);
                    log = JsonConvert.DeserializeObject<LogModel>(logHub);
                    if (log != null)
                    {
                        if (!Directory.Exists(log.path)) Directory.CreateDirectory(log.path);
                        //FileInfo fileInfo = new FileInfo(Path.Combine(log.path, log.Title));
                        //if (fileInfo.Length > 5000)
                        //{
                        //    var title = log.Title.Split(".");
                        //    File.Move(log.Title, title[0] + "" + title[1]);
                        //}
                        File.AppendAllText(Path.Combine(log.path, log.Title), log.Con);
                    }
                }
                catch (Exception ex)
                {
                    var path = $"D:\\WCSLogs\\{DateTime.Now.yyyyMMdd()}\\日志处理进程";
                    if (!Directory.Exists(path)) Directory.CreateDirectory(path);
                    File.AppendAllText(Path.Combine($"D:\\WCSLogs\\{DateTime.Now.yyyyMMdd()}\\日志处理进程", "Error.txt"), $"{JsonConvert.SerializeObject(log)}--{ex.Message}--{ex.StackTrace}\n");
                }
            }
        }
    }

    public class LogModel
    {
        public string path { get; set; }
        public string Title { get; set; }

        public string Con { get; set; }
    }

    /// <summary>
    ///  业务报警
    /// </summary>
    public class BusinessAlarm
    {
        /// <summary>
        ///  业务名称
        /// </summary>
        public string BusinessName { get; set; }

        /// <summary>
        ///  设备号
        /// </summary>
        public string DevNo { get; set; }

        /// <summary>
        ///  内容
        /// </summary>
        public string Con { get; set; }

        /// <summary>
        ///  时间
        /// </summary>
        public DateTime Time { get; set; }
    }
}
